@@ -29,7 +29,7 @@
 //! </ul>
 //!
 //! \details The \p TF_ prefix means an implementation using trapdoor functions on integers.
-//! \details The \p DL_ prefix means an implementation using group operations (in groups where discrete log is hard).
+//! \details The \p DL_ prefix means an implementation using group operations in groups where discrete log is hard.
 
 #ifndef CRYPTOPP_PUBKEY_H
 #define CRYPTOPP_PUBKEY_H
@@ -55,6 +55,12 @@
 // VC60 workaround: this macro is defined in shlobj.h and conflicts with a template parameter used in this file
 #undef INTERFACE
 
+#if defined(__SUNPRO_CC)
+# define MAYBE_RETURN(x) return x
+#else
+# define MAYBE_RETURN(x) CRYPTOPP_UNUSED(x)
+#endif
+
 NAMESPACE_BEGIN(CryptoPP)
 
 //! \class TrapdoorFunctionBounds
@@ -321,7 +327,8 @@ public:
 
 // ********************************************************
 
-typedef std::pair<const byte *, size_t> HashIdentifier;
+// Typedef change due to Clang, http://github.com/weidai11/cryptopp/issues/300
+typedef std::pair<const byte *, unsigned int> HashIdentifier;
 
 //! \class PK_SignatureMessageEncodingMethod
 //! \brief Interface for message encoding method for public key signature schemes.
@@ -359,7 +366,7 @@ public:
 		CRYPTOPP_UNUSED(hash);CRYPTOPP_UNUSED(recoverableMessage); CRYPTOPP_UNUSED(recoverableMessageLength);
 		CRYPTOPP_UNUSED(presignature); CRYPTOPP_UNUSED(presignatureLength); CRYPTOPP_UNUSED(semisignature);
 		if (RecoverablePartFirst())
-			assert(!"ProcessRecoverableMessage() not implemented");
+			CRYPTOPP_ASSERT(!"ProcessRecoverableMessage() not implemented");
 	}
 
 	virtual void ComputeMessageRepresentative(RandomNumberGenerator &rng,
@@ -1308,6 +1315,7 @@ public:
 	{
 		CRYPTOPP_UNUSED(params); CRYPTOPP_UNUSED(publicKey); CRYPTOPP_UNUSED(r); CRYPTOPP_UNUSED(s);
 		throw NotImplemented("DL_ElgamalLikeSignatureAlgorithm: this signature scheme does not support message recovery");
+		MAYBE_RETURN(Integer::Zero());
 	}
 	virtual size_t RLen(const DL_GroupParameters<T> &params) const
 		{return params.GetSubgroupOrder().ByteCount();}
@@ -1413,7 +1421,7 @@ public:
 	//! \returns maximum recoverable length based on signature length, in bytes
 	//! \details this function is not implemented and always returns 0.
 	size_t MaxRecoverableLengthFromSignatureLength(size_t signatureLength) const
-		{CRYPTOPP_UNUSED(signatureLength); assert(false); return 0;}	// TODO
+		{CRYPTOPP_UNUSED(signatureLength); CRYPTOPP_ASSERT(false); return 0;}	// TODO
 
 	//! \brief Determines if the scheme is probabilistic
 	//! \returns true if the scheme is probabilistic, false otherwise
@@ -1999,7 +2007,7 @@ public:
 			return params.ExponentiateElement(publicElement, privateExponent*params.GetCofactor());
 		else
 		{
-			assert(COFACTOR_OPTION::ToEnum() == NO_COFACTOR_MULTIPLICTION);
+			CRYPTOPP_ASSERT(COFACTOR_OPTION::ToEnum() == NO_COFACTOR_MULTIPLICTION);
 
 			if (!validateOtherPublicKey)
 				return params.ExponentiateElement(publicElement, privateExponent);
